@@ -1,17 +1,16 @@
-'''Concurrent server and message queues'''
-VERSION = (0, 2, 1)
+'''Event driven concurrent framework for Python'''
+VERSION = (0, 3, 0, 'final', 0)
 
-def get_version():
-    return '.'.join(map(str,VERSION))
+from .utils.version import get_version
 
-__version__   = get_version()
+__version__   = version = get_version(VERSION)
 __license__   = "BSD"
 __author__    = "Luca Sbardella"
 __contact__   = "luca.sbardella@gmail.com"
 __homepage__  = "https://github.com/quantmind/pulsar"
 __docformat__ = "restructuredtext"
 CLASSIFIERS  = [
-                'Development Status :: 1 - Planning',
+                'Development Status :: 3 - Alpha',
                 'Environment :: Web Environment',
                 'Intended Audience :: Developers',
                 'License :: OSI Approved :: BSD License',
@@ -20,13 +19,22 @@ CLASSIFIERS  = [
                 'Programming Language :: Python :: 2.6',
                 'Programming Language :: Python :: 2.7',
                 'Programming Language :: Python :: 3',
-                'Topic :: Utilities'
+                'Programming Language :: Python :: 3.1',
+                'Programming Language :: Python :: 3.2',
+                'Programming Language :: Python :: 3.3',
+                'Topic :: Internet',
+                'Topic :: Utilities',
+                'Topic :: Software Development :: Libraries :: Python Modules',
+                'Topic :: Internet :: WWW/HTTP',
+                'Topic :: Internet :: WWW/HTTP :: WSGI',
+                'Topic :: Internet :: WWW/HTTP :: WSGI :: Server',
+                'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
                 ]
 
 from .utils.log import *
 
 DEFAULT_PORT = 8060
-SERVER_SOFTWARE = "python-{0}/{1}".format(SERVER_NAME,get_version())
+SERVER_SOFTWARE = "python-{0}/{1}".format(SERVER_NAME, version)
 
 class NOT_DONE(object):
     pass
