@@ -3,10 +3,11 @@ import os
 import sys
 import traceback
 import random
-from inspect import isgenerator
+from inspect import isgenerator, isfunction
 
 import pulsar
-from pulsar import Empty, make_async, is_failure, async_pair, Failure
+from pulsar import Empty, make_async, is_failure, async_pair, Failure,\
+                     HaltServer
 from pulsar.utils.py2py3 import execfile, pickle
 from pulsar.utils.importer import import_module
 from pulsar.utils.log import LogInformation
@@ -16,6 +17,7 @@ from pulsar.utils.config import Setting
 
 __all__ = ['Worker',
            'Application',
+           'ApplicationHandlerMixin',
            'ApplicationMonitor',
            'WorkerRequest',
            'Response',
@@ -53,7 +55,7 @@ class Response(object):
         pass
 
 
-class ResponseError(pulsar.PulsarException,Response):
+class ResponseError(pulsar.PulsarException, Response):
     
     def __init__(self, request, failure):
         pulsar.Response.__init__(self, request)
@@ -74,8 +76,45 @@ def make_response(request, response, err = None):
     return response
 
 
-class HandlerMixin(object):
-      
+def halt_server(f):
+    '''Halt server decorator'''
+    def _(self, *args, **kwargs):
+        try:
+            return f(self, *args, **kwargs)
+        except Exception as e:
+            if self.app.can_kill_arbiter:
+                msg = 'Unhadled exception in {0} application: {1}'\
+                        .format(self.app.name, e)
+                self.log.critical(msg, exc_info = True)
+                raise HaltServer(msg)
+            else:
+                raise
+    _.__name__ = f.__name__
+    _.__doc__ = f.__doc__
+    return _
+            
+
+class ApplicationHandlerMixin(object):
+    '''A mixin for both :class:`Worker` and :class:`ApplicationMonitor`.
+It implements :meth:`handle_request` and :meth:`close_response`
+used for by the :class:`Application` for handling requests and
+sending back responses.
+'''
+    def on_message(self, message):
+        self.app.on_actor_message(message)
+        
+    def on_message_processed(self, message, result):
+        self.app.on_actor_message_processed(message, result)
+        
+    def handle_task(self):
+        if self.information.log():
+            self.log.info('Processed {0} requests'.format(self.nr))
+        try:
+            self.cfg.worker_task(self)
+        except:
+            pass
+        self.app.worker_task(self)
+        
     def _response_generator(self, request):
         try:
             self.cfg.pre_request(self, request)
@@ -138,11 +177,10 @@ After obtaining the result from the
         self.setlog()
 
 
-class Worker(HandlerMixin,pulsar.Actor):
+class Worker(ApplicationHandlerMixin,pulsar.Actor):
     """\
-Base class for a :class:`Actor` serving a :class:`Application`.
-It provides two new functions :meth:`handle_request` and :meth:`handle_response`
-used for by the application for handling requests and sending back responses.
+An :class:`Actor` class for serving an :class:`Application`.
+It provides two new methods inherited from :class:`ApplicationHandlerMixin`.
     
 .. attribute:: app
 
@@ -175,9 +213,7 @@ used for by the application for handling requests and sending back responses.
             pass
     
     def on_task(self):
-        if self.information.log():
-            self.log.info('Processed {0} requests'.format(self.nr))
-        self.app.worker_task(self)
+        self.handle_task()
     
     def on_stop(self):
         self.app.worker_stop(self)
@@ -194,9 +230,9 @@ used for by the application for handling requests and sending back responses.
         return self.app.on_info(self,info)
 
 
-class ApplicationMonitor(HandlerMixin,pulsar.Monitor):
-    '''A spcialized :class:`Monitor` implementation for managing
-pulsar applications (subclasses of :class:`Application`).
+class ApplicationMonitor(ApplicationHandlerMixin, pulsar.Monitor):
+    '''A specialized :class:`Monitor` implementation for managing
+pulsar subclasses of :class:`Application`.
 '''
     # For logging name
     _class_code = 'appmonitor'
@@ -219,19 +255,19 @@ pulsar applications (subclasses of :class:`Application`).
         super(ApplicationMonitor,self).on_init(**kwargs)
         self.app.monitor_init(self)
     
-    # Delegates Callbacks to the application    
+    # Delegates Callbacks to the application
+    @halt_server    
     def on_start(self):
         self.app.monitor_start(self)
         if not self.cfg.workers:
             self.app.worker_start(self)            
         
+    @halt_server
     def monitor_task(self):
         self.app.monitor_task(self)
         # There are no workers, the monitor do their job
         if not self.cfg.workers:
-            if self.information.log():
-                self.log.info('Processed {0} requests'.format(self.nr))
-            self.app.worker_task(self)
+            self.handle_task()
             
     def on_stop(self):
         stp = make_async(self.app.monitor_stop(self))
@@ -296,7 +332,7 @@ its duties.
 
     A string indicating the application namespace for configuration parameters.
     
-    Default ``None``
+    Default: ``None``.
     
 .. attribute:: callable
 
@@ -311,7 +347,7 @@ its duties.
 
     dictionary of default configuration parameters.
     
-    Default: ``{}``
+    Default: ``{}``.
     
 .. attribute:: mid
 
@@ -322,6 +358,15 @@ its duties.
 
     full path of the script which starts the application or ``None``.
     If supplied it is used to setup the python path
+    
+.. attribute:: can_kill_arbiter
+
+    If ``True``, an unhandled error in the application will shut down the
+    :class:`pulsar.Arbiter`. Check the :meth:`ApplicationMonitor.monitor_task`
+    method for implementation.
+    
+    Default: ``False``.
+    
 """
     cfg = {}
     _name = None
@@ -330,6 +375,7 @@ its duties.
     app = None
     config_options_include = None
     config_options_exclude = None
+    can_kill_arbiter = False
     monitor_class = ApplicationMonitor
     
     def __init__(self,
@@ -339,8 +385,19 @@ its duties.
                  epilog = None,
                  argv = None,
                  script = None,
+                 version = None,
+                 can_kill_arbiter = None,
                  **params):
+        '''Initialize a new :class:`Application` and add its
+:class:`ApplicationMonitor` to the class:`pulsar.Arbiter`.
+
+:parameter version: Optional version number of the application.
+
+    Default: ``pulsar.__version__``
+'''
         self.description = description or self.description
+        if can_kill_arbiter is not None:
+            self.can_kill_arbiter = bool(can_kill_arbiter) 
         self.epilog = epilog or self.epilog
         self._name = name or self._name or self.__class__.__name__.lower()
         self.script = script
@@ -348,7 +405,7 @@ its duties.
         nparams = self.cfg.copy()
         nparams.update(params)
         self.callable = callable
-        self.load_config(argv,**nparams)
+        self.load_config(argv,version=version,**nparams)
         self.configure_logging()
         if self.on_config() is not False:
             arbiter = pulsar.arbiter(self.cfg.daemon)
@@ -438,7 +495,8 @@ By default it returns ``None``.'''
     def add_timeout(self, deadline, callback):
         self.arbiter.ioloop.add_timeout(deadline, callback)
               
-    def load_config(self, argv, parse_console = True, **params):
+    def load_config(self, argv, version = None,
+                    parse_console = True, **params):
         '''Load the application configuration from a file and/or
 from the command line. Called during application initialization.
 
@@ -455,6 +513,7 @@ The parameters overrriding order is the following:
 '''
         self.cfg = pulsar.Config(self.description,
                                  self.epilog,
+                                 version,
                                  self.app,
                                  self.config_options_include,
                                  self.config_options_exclude)
@@ -499,13 +558,14 @@ The parameters overrriding order is the following:
         
         # Load up the config file if its found.
         if config and os.path.exists(config):
-            cfg = {
-                "__builtins__": __builtins__,
-                "__name__": "__config__",
-                "__file__": config,
-                "__doc__": None,
-                "__package__": None
-            }
+            #cfg = {
+            #    "__builtins__": __builtins__,
+            #    "__name__": "__config__",
+            #    "__file__": config,
+            #    "__doc__": None,
+            #    "__package__": None
+            #}
+            cfg = {}
             try:
                 execfile(config, cfg, cfg)
             except Exception:
@@ -524,6 +584,13 @@ The parameters overrriding order is the following:
                         sys.stderr.write("Invalid value for %s: %s\n\n"\
                                           % (k, v))
                         raise
+                    else:
+                        if isfunction(v):
+                            if v.__name__ in globals():
+                                v = globals()[v.__name__]
+                                self.cfg.set(k.lower(), v)
+                            else:
+                                globals()[v.__name__] = v
             
         # Update the configuration with any command line settings.
         if opts:
@@ -613,6 +680,12 @@ The application is now in the arbiter but has not yet started.'''
         '''Callback by :class:`ApplicationMonitor` at each event loop'''
         pass
     
+    def on_actor_message(self, message):
+        pass
+        
+    def on_actor_message_processed(self, message, result):
+        pass
+    
     def start(self):
         '''Start the application if it wasn't already started.'''
         arbiter = pulsar.arbiter()
