@@ -39,7 +39,7 @@ class JsonToolkit(object):
 
 
 class JSONRPC(RpcHandler):
-    '''Base class for JSON-RPC services.
+    '''A :class:`RpcHandler` for class for JSON-RPC services.
 Design to comply with the `JSON-RPC 2.0`_ Specification.
 
 .. _`JSON-RPC 2.0`: http://groups.google.com/group/json-rpc/web/json-rpc-2-0'''
@@ -49,6 +49,8 @@ Design to comply with the `JSON-RPC 2.0`_ Specification.
     _json = JsonToolkit
         
     def get_method_and_args(self, data):
+        '''Overrides the :meth:`RpcHandler:get_method_and_args` to obtain
+method data from the JSON *data* string.'''
         if not isinstance(data,dict):
             data = self._json.loads(data)
         method  = data.get('method',None)
@@ -180,7 +182,7 @@ usage is simple::
         start = default_timer()
         for t in r:
             func(*args, **kwargs)
-        return default_timer() - start            
+        return default_timer() - start
         
     def _get_data(self, *args, **kwargs):
         func_name = self.__name
@@ -246,7 +248,8 @@ usage is simple::
     
     
 class LocalJsonProxy(JsonProxy):
-    
+    '''A proxy class to use when accessing the rpc within the rpc application
+domain.'''
     def setup(self, handler = None, environ = None, **kwargs):
         self.local['handler'] = handler
         self.local['environ'] = environ
