@@ -1,11 +1,9 @@
-from datetime import datetime, date
+from datetime import datetime, date, timedelta
 from hashlib import sha1
 import logging
 import inspect
 
 from pulsar.utils.py2py3 import iteritems
- 
-
 from pulsar.utils.tools import gen_unique_id
 
 
@@ -103,7 +101,7 @@ class Job(JobBase):
 .. attribute:: abstract
 
     If set to ``True`` (default is ``False``), the Job won't be registered
-    with the Job registry.
+    with the :class:`JobRegistry`.
     
 .. attribute:: autoregister
 
@@ -117,16 +115,29 @@ class Job(JobBase):
 .. attribute:: timeout
 
     An instance of a datetime.timedelta or ``None``. If set, it represents the
-    time lag after which a task whch has not been processed expires.
+    time lag after which a task which did not start expires.
     
-    Default ``None``.
+    Default: ``None``.
     
 .. attribute:: can_overlap
 
-    Boolean indicating if this job can generate overlapping tasks with same
-    input parameters.
+    Boolean indicating if this job can generate overlapping tasks. It can
+    also be a callable which accept the same input parameters as the job
+    callable function.
     
-    Default ``True``.
+    Default: ``True``.
+    
+.. attribute:: doc_syntax
+
+    The doc string syntax.
+    
+    Default: ``markdown``
+    
+.. attribute:: loglevel
+
+    Level of task logging
+    
+    Default: ``None``
     
 .. attribute:: logger
 
@@ -137,7 +148,9 @@ class Job(JobBase):
     type = "regular"
     timeout = None
     expires = None
+    loglevel = None
     logformatter = None
+    doc_syntax = 'markdown'
     can_overlap = True
     _ack = True
         
@@ -157,7 +170,10 @@ This can be overridden by Job implementation.
     
 Called by the :attr:`TaskQueue.scheduler` when creating a new task.
 '''
-        if self.can_overlap:
+        can_overlap = self.can_overlap
+        if hasattr(can_overlap,'__call__'):
+            can_overlap = can_overlap(*args, **kwargs)
+        if can_overlap:
             return create_task_id()
         else:
             suffix = ''
@@ -182,6 +198,24 @@ task is aborted.'''
  behaviour can change at runtime.'''
         return self._ack
     
+    def send_to_queue(self, consumer, jobname, ack = True, **kwargs):
+        '''Send a new task request to the :class:`TaskQueue`
+from within another :class:`Task`.
+This allow for tasks acting as tasks factories.
+
+:parameter consumer: The :class:`TaskConsumer` handling the :class:`Task`.
+:parameter jobname: The name of the :class:`Job` to run.
+:parameter ack: if acknowledgment is needed. Default ``True``.
+:parameter kwargs: key-valued parameters for the task.
+:rtype: an :class:`pulsar.ActorMessage`.
+'''
+        worker = consumer.worker
+        oper = "addtask" if ack else "addtask_noack"
+        from_task = consumer.task.id
+        return worker.monitor.send(worker, oper, jobname,
+                                   {'from_task':from_task},
+                                   **kwargs)
+    
 
 class PeriodicJob(Job):
     '''A periodic :class:`Job` implementation.'''
@@ -196,8 +230,8 @@ It controls when the periodic Job is run.'''
     def __init__(self, run_every = None):
         self.run_every = run_every or self.run_every
         if self.run_every is None:
-            raise NotImplementedError("Periodic Jobs must have\
- a run_every attribute set.")
+            raise NotImplementedError('Periodic Jobs must have\
+ a run_every attribute set, "{0}" does not have one'.format(self.name))
    
     def is_due(self, last_run_at):
         """Returns tuple of two items ``(is_due, next_time_to_run)``,
