@@ -51,8 +51,8 @@ class Schedule(object):
 
 
 class SchedulerEntry(object):
-    """A class which can be used as a schedule entry in a :class:`Scheduler` instance.
-    """
+    """A class which can be used as a schedule entry in a
+:class:`Scheduler` instance."""
     name = None
     '''Task name'''
     schedule = None
@@ -69,6 +69,10 @@ class SchedulerEntry(object):
         self.last_run_at = last_run_at or datetime.now()
         self.total_run_count = total_run_count or 0
 
+    def __repr__(self):
+        return self.name
+    __str__ = __repr__
+    
     @property
     def scheduled_last_run_at(self):
         '''The scheduled last run datetime. This is different from :attr:`last_run_at` only when :attr:`anchor` is set.'''
@@ -76,7 +80,8 @@ class SchedulerEntry(object):
         anchor = self.anchor
         if last_run_at and anchor:
             run_every = self.run_every
-            times = int(timedelta_seconds(last_run_at - anchor)/timedelta_seconds(run_every))
+            times = int(timedelta_seconds(last_run_at - anchor)\
+                            /timedelta_seconds(run_every))
             if times:
                 anchor += times*run_every
                 while anchor <= last_run_at:
@@ -122,46 +127,23 @@ and task scheduling."""
     def entries(self):
         return self._entries
         
-    def make_request(self, job_name, targs = None, tkwargs = None,
-                     expiry = None, **kwargs):
-        '''Create a new task request.
+    def queue_task(self, monitor, jobname, targs = None, tkwargs = None,
+                   **params):
+        '''Create a new :class:`Task` which may or may not queued.
 
-:parameter job_name: the name of a :class:`Job` registered
-    with the application.
-:parameter targs: optional tuple of arguments for the task.
-:parameter tkwargs: optional dictionary of arguments for the task.
+:parameter monitor: the :class:`pulsar.ApplicationMonitor` running the
+    :class:`TaskQueue` application.
+:parameter jobname: the name of a :class:`Job` registered
+    with the :class:`TaskQueue` application.
+:parameter targs: optional tuple used for the positional arguments in the
+    task callable.
+:parameter tkwargs: optional dictionary used for the key-valued arguments
+    in the task callable.
+:parameter params: Additional parameters to be passed to the :class:`Task`
+    constructor (not its callable function).
     
 :rtype: an instance of :class:`Task`'''
-        if job_name in registry:
-            TaskFactory = self.TaskFactory
-            job = registry[job_name]
-            targs = targs or EMPTY_TUPLE
-            tkwargs = tkwargs or EMPTY_DICT
-            id = job.make_task_id(targs,tkwargs)
-            task = TaskFactory.get_task(id, remove = True)
-            if task:
-                return task.to_queue()
-            else:
-                if job.name in self.entries:
-                    self.entries[job.name].next()
-                time_executed = datetime.now()
-                expiry = get_datetime(expiry, time_executed)
-                if not expiry and job.timeout:
-                    expiry = time_executed + job.timeout
-                task = TaskFactory(id = id, name = job.name,
-                                   time_executed = time_executed,
-                                   expiry = expiry, args = targs,
-                                   kwargs = tkwargs,
-                                   status = PENDING, **kwargs)
-                return task.to_queue()
-        else:
-            raise TaskNotAvailable(job_name)
-        
-    def queue_task(self, monitor, job_name, targs = None, tkwargs = None,
-                   **kwargs):
-        '''Put a new task in the task queue. This function invokes
-:meth:`make_request` and check that the task can be inserted into the queue.'''
-        task = self.make_request(job_name, targs, tkwargs, **kwargs)
+        task = self._make_request(jobname, targs, tkwargs, **params)
         if task.needs_queuing():
             task._queued = True
             monitor.put(task.serialize_for_queue())
@@ -170,18 +152,20 @@ and task scheduling."""
             self.log.info('task {0} already requested. Abort request.'\
                                   .format(task))
         return task
-
-    def tick(self, monitor, now = None):
+    
+    def tick(self, monitor, now=None):
         '''Run a tick, that is one iteration of the scheduler.
 Executes all due tasks calculate the time in seconds to wait before
 running a new :meth:`tick`. For testing purposes a :class:`datetime.datetime`
 value ``now`` can be passed.'''
+        # First we check for tasks which have timed out
+        self.TaskFactory.check_unready_tasks()
         remaining_times = []
         try:
             for entry in itervalues(self._entries):
-                is_due, next_time_to_run = entry.is_due(now = now)
+                is_due, next_time_to_run = entry.is_due(now=now)
                 if is_due:
-                    self.queue_task(monitor,entry.name)
+                    self.queue_task(monitor, entry.name)
                 if next_time_to_run:
                     remaining_times.append(next_time_to_run)
         except RuntimeError:
@@ -210,7 +194,9 @@ value ``now`` can be passed.'''
             if name not in registry:
                 continue
             job = registry[name]
-            d = {'doc':job.__doc__, 'type':job.type}
+            d = {'doc':job.__doc__,
+                 'doc_syntax':job.doc_syntax,
+                 'type':job.type}
             if name in self.entries:
                 entry = self.entries[name]
                 _,next_time_to_run = self.next_scheduled(name)
@@ -244,4 +230,38 @@ value ``now`` can be passed.'''
                 return (next_entry.name,max(next_time,0))
             
         return (jobname,None)
-        
+    
+    ############################################################################
+    ##    PRIVATE METHODS
+    ############################################################################
+    
+    def _make_request(self, jobname, targs = None, tkwargs = None,
+                     expiry = None, **params):
+        if jobname in registry:
+            TaskFactory = self.TaskFactory
+            job = registry[jobname]
+            targs = targs or EMPTY_TUPLE
+            tkwargs = tkwargs or EMPTY_DICT
+            id = job.make_task_id(targs, tkwargs)
+            task = TaskFactory.get_task(id, remove=True)
+            if task:
+                return task.to_queue(self)
+            else:
+                if job.name in self.entries:
+                    self.entries[job.name].next()
+                time_executed = datetime.now()
+                expiry = get_datetime(expiry, time_executed)
+                if not expiry and job.timeout:
+                    expiry = time_executed + job.timeout
+                task = TaskFactory(id = id, name = job.name,
+                                   time_executed = time_executed,
+                                   expiry = expiry,
+                                   args = targs,
+                                   kwargs = tkwargs,
+                                   status = PENDING,
+                                   **params)
+                task.on_created(self)
+                return task.to_queue(self)
+        else:
+            raise TaskNotAvailable(jobname)
+        
\ No newline at end of file
