@@ -72,6 +72,10 @@ from .loader import *
 from .utils import *
 
 
+class ExitTest(Exception):
+    pass
+
+
 class TestSuite(pulsar.Application):
     '''An asynchronous test suite which works like a task queue where each task
 is a group of tests specified in a test class.
@@ -87,7 +91,7 @@ is a group of tests specified in a test class.
     files and directories, while it will look into the example directory for all
     files or directories matching ``tests``.
     
-    Alternatively it can ba a callable returning the iterable over modules. The
+    Alternatively it can be a callable returning the iterable over modules. The
     callable must accept one positional argument, the instance of the test
     suite::
     
@@ -106,6 +110,7 @@ is a group of tests specified in a test class.
                               'worker_class','debug','task_queue_factory',
                               'http_proxy','http_client','http_py_parser')
     default_logging_level = None
+    can_kill_arbiter = True
     cfg = {'workers':1, 'loglevel':'none'}
     
     def handler(self):
@@ -133,7 +138,9 @@ configuration and plugins.'''
             r = unittest.TextTestRunner()
             stream = r.stream
             runner = TestRunner(self.plugins,stream,result_class)
-            runner.configure(self.cfg)
+            abort_message = runner.configure(self.cfg)
+            if abort_message:
+                raise ExitTest(str(abort_message))
             self.local['runner'] = runner
         return self.local['runner'] 
             
@@ -146,7 +153,7 @@ configuration and plugins.'''
             self.plugins = ()
             
         # Create a runner and configure it
-        runner = self.runner
+        runner = self.runner            
         
         if not modules:
             modules = ((None,'tests'),)
@@ -165,10 +172,13 @@ configuration and plugins.'''
                       .format(s,', '.join(tags)))
             else:
                 print('\nAll test labels\n')
-            for tag,mod in loader.testmodules(tags):
-                doc = mod.__doc__
-                if doc:
-                    tag = '{0} - {1}'.format(tag,doc)
+            def _tags():
+                for tag,mod in loader.testmodules(tags):
+                    doc = mod.__doc__
+                    if doc:
+                        tag = '{0} - {1}'.format(tag,doc)
+                    yield tag
+            for tag in sorted(_tags()):
                 print(tag)
             print('\n')
             return False
@@ -209,4 +219,4 @@ configuration and plugins.'''
         
     def actor_test_result(self, sender, worker, result):
         self.runner.add(result)
-    
\ No newline at end of file
+
