@@ -1,6 +1,5 @@
 import sys
 import io
-import unittest
 import pickle
 from inspect import istraceback, isclass
 
@@ -113,6 +112,10 @@ class TestRequest(WorkerRequest):
 .. attribute:: testcls
 
     A :class:`unittest.TestCase` class to be run on this request.
+    
+.. attribute:: tag
+
+    A string indicating the tag associated with :attr:`testcls`.
 '''
     def __init__(self, testcls, tag):
         self.testcls = testcls
@@ -123,15 +126,19 @@ class TestRequest(WorkerRequest):
     __str__ = __repr__
         
     def run(self, worker):
-        '''Run all tests from the :attr:`testcls`. First it checks if 
-a class method ``setUpClass`` is defined. If so it runs it.'''
+        '''Run all test functions from the :attr:`testcls` using the
+following algorithm:
+
+* Run the class method ``setUpClass`` of :attr:`testcls` of if defined.
+* Call :meth:`run_test` for each test functions in :attr:`testcls`
+* Run the class method ``tearDownClass`` of :attr:`testcls` if defined.  
+'''
         # Reset the runner
         worker.app.local.pop('runner', None)
         runner = worker.app.runner
-        loader = unittest.TestLoader()
         testcls = self.testcls
         testcls.tag = self.tag
-        all_tests = loader.loadTestsFromTestCase(testcls)
+        all_tests = runner.loadTestsFromTestCase(testcls)
         
         if all_tests.countTestCases():
             testcls.worker = worker
@@ -157,7 +164,7 @@ a class method ``setUpClass`` is defined. If so it runs it.'''
                     runner.stopTest(test)
                 
             if end:
-                result,outcome = end()
+                result, outcome = end()
                 yield result
                 self.add_failure(test, runner, outcome.result)
             
@@ -170,50 +177,65 @@ a class method ``setUpClass`` is defined. If so it runs it.'''
         worker.monitor.send(worker,'test_result',runner.result)
         
     def run_test(self, test, runner):
-        '''Run a *test* function.'''
-        testMethod = getattr(test, test._testMethodName)
-        if (getattr(test.__class__, "__unittest_skip__", False) or
-            getattr(testMethod, "__unittest_skip__", False)):
-            # If the class or method was skipped.
-            try:
-                reason = (getattr(test.__class__, '__unittest_skip_why__', '')
-                            or getattr(testMethod, '__unittest_skip_why__', ''))
+        '''\
+Run a *test* function using the following algorithm
+        
+* Run :meth:`_pre_setup` method if available in :attr:`testcls`.
+* Run :meth:`setUp` method in :attr:`testcls`.
+* Run the test function.
+* Run :meth:`tearDown` method in :attr:`testcls`.
+* Run :meth:`_post_teardown` method if available in :attr:`testcls`.
+'''
+        try:
+            success = True
+            testMethod = getattr(test, test._testMethodName)
+            if (getattr(test.__class__, "__unittest_skip__", False) or
+                getattr(testMethod, "__unittest_skip__", False)):
+                reason = (getattr(test.__class__,
+                                  '__unittest_skip_why__', '') or
+                          getattr(testMethod,
+                                  '__unittest_skip_why__', ''))
                 runner.addSkip(test, reason)
-            except:
-                pass
-            raise StopIteration
-        
-        success = True
-        if hasattr(test,'_pre_setup'):
-            result, outcome = run_on_arbiter(test,test._pre_setup)()
-            yield result
-            success = not self.add_failure(test, runner, outcome.result)
-        
-        if success:
-            result, outcome = run_on_arbiter(test,test.setUp)()
-            yield result
-            if not self.add_failure(test, runner, outcome.result):
-                # Here we perform the actual test
-                result, outcome = run_on_arbiter(test,testMethod,istest=True)()
+                raise StopIteration()
+            
+            if hasattr(test,'_pre_setup'):
+                result, outcome = run_on_arbiter(test,test._pre_setup)()
                 yield result
                 success = not self.add_failure(test, runner, outcome.result)
-                if success:
-                    test.result = outcome.result
-                result, outcome = run_on_arbiter(test,test.tearDown)()
+            
+            if success:
+                result, outcome = run_on_arbiter(test,test.setUp)()
+                yield result
+                if not self.add_failure(test, runner, outcome.result):
+                    # Here we perform the actual test
+                    result, outcome = run_on_arbiter(test,testMethod,
+                                                     istest=True)()
+                    yield result
+                    success = not self.add_failure(test, runner, outcome.result)
+                    if success:
+                        test.result = outcome.result
+                    result, outcome = run_on_arbiter(test,test.tearDown)()
+                    yield result
+                    if self.add_failure(test, runner, outcome.result):
+                        success = False
+                else:
+                    success = False
+                    
+            if hasattr(test,'_post_teardown'):
+                result, outcome = run_on_arbiter(test,test._post_teardown)()
                 yield result
                 if self.add_failure(test, runner, outcome.result):
                     success = False
-            else:
-                success = False
-                
-        if hasattr(test,'_post_teardown'):
-            result, outcome = run_on_arbiter(test,test._post_teardown)()
-            yield result
-            if self.add_failure(test, runner, outcome.result):
-                success = False
-    
+        
+        except StopIteration:
+            success = False
+        except Exception as e:
+            self.add_failure(test, runner, e)
+            success = False
+        
         if success:
             runner.addSuccess(test)
+        
 
     def add_failure(self, test, runner, failure):
         failure = as_failure(failure)
