@@ -8,6 +8,9 @@ class HttpClient(test.Plugin):
     '''A test plugin which creates a fake http client for your
  wsgi handler.'''
     def getTest(self, test):
-        wsgi = getattr(test,'wsgi_handler',None)
+        test.client = lambda : self._get_client(test)
+            
+    def _get_client(self, test):
+        wsgi = getattr(test, 'wsgi_handler', None)
         if wsgi:
-            test.client = HttpTestClientRequest(wsgi)
\ No newline at end of file
+            return HttpTestClientRequest(wsgi())
\ No newline at end of file
