@@ -1,5 +1,7 @@
 import sys
 import traceback
+import unittest
+import logging
 from inspect import istraceback
 from copy import deepcopy
 
@@ -10,23 +12,31 @@ __all__ = ['TestObject',
            'TestResult']
 
 
+LOGGER = logging.getLogger('pulsar.apps.test')
 STDOUT_LINE = '\nStdout:\n%s'
 STDERR_LINE = '\nStderr:\n%s'
 
 
 class TestObject(object):
-    '''Interface for all test classes which are part of of the
-:class:`TestRunner`.'''
+    '''Interface for all classes which are part of of the :class:`TestRunner`,
+including :class:`TestRunner` itself, :class:`TestResult`
+and :class:`Plugin`.'''
     descriptions = None
     
     def configure(self, cfg):
         '''Configure the *instance*. This method is called once just after
-construction.
+construction of a :class:`TestRunner` and can be used to configure the
+plugin. If it returns something other than ``None``
+(for example an abort message)
+it will stop the configuration of all subsequent plugins and quit the test.
 
-:paremeter cfg: instance of :class:`pulsar.Config`.
+:parameter cfg: instance of :class:`pulsar.Config`.
 '''
         pass
     
+    def loadTestsFromTestCase(self, cls):
+        pass
+    
     def on_start(self):
         '''Called once by :class:`TestSuite` just before it starts running tests.'''
         pass
@@ -288,7 +298,7 @@ class TestResult(TestObject):
             else:
                 msgLines = traceback.format_exception(exctype, value, tb)
         else:
-            msgLines = tb
+            msgLines = tb or []
 
         return ''.join(msgLines)
 
@@ -322,13 +332,23 @@ class TestRunner(TestResultProxy):
         self.plugins.append(stream)
         self.stream = stream
         self.result = result
+        self.loader = unittest.TestLoader()
         
     def configure(self, cfg):
         self.cfg = cfg
         for p in self.plugins:
-            if p.configure(cfg):
-                break
-            
+            c = p.configure(cfg)
+            if c:
+                return c 
+    
+    def loadTestsFromTestCase(self, cls):
+        all_tests = self.loader.loadTestsFromTestCase(cls)
+        for p in self.plugins:
+            c = p.loadTestsFromTestCase(cls)
+            if c is not None:
+                return c
+        return all_tests
+        
     def on_start(self):
         '''Called just before the test suite starts running tests.'''
         for p in self.plugins:
@@ -338,8 +358,13 @@ class TestRunner(TestResultProxy):
     def on_end(self):
         '''Called just before the test suite starts running tests.'''
         for p in self.plugins:
-            if p.on_end():
-                break
+            try:
+                if p.on_end():
+                    break
+            except:
+                LOGGER.critical('Unhandled exception while calling method'\
+                                ' "on_end" of plugin {0}'.format(p),
+                                exc_info=True)
         
     def add(self, result):
         self.result.add(result)
