@@ -1,6 +1,6 @@
 """
-Pulsar is shipped with a HTTP applications which conforms the python
-web server gateway interface (WSGI_).
+Pulsar is shipped with an HTTP :class:`pulsar.apps.Application` which conforms
+with the python web server gateway interface (WSGI_).
 
 The application can be used in conjunction with several web frameworks
 as well as the :ref:`pulsar RPC middleware <apps-rpc>` and
@@ -53,6 +53,7 @@ It can be configured to run as a multiprocess or a multithreaded server.'''
         return self.wsgi_handler(callable)
     
     def wsgi_handler(self, hnd, resp_middleware = None):
+        '''Build the wsgi handler'''
         if not isinstance(hnd,WsgiHandler):
             if not isinstance(hnd,(list,tuple)):
                 hnd = [hnd]
@@ -114,14 +115,16 @@ requests in the threaded workers.'''
                                        multiprocess = mp)
         # Create the response object
         response = HttpResponse(request)
-        # Get the data from the WSGI handler
-        try:
-            data = worker.app_handler(environ, response.start_response)
-        except Exception as e:
-            worker.log.critical('Unhandled WSGI exception', exc_info = True)
-            # An unhadled exception in the handler
-            data = cfg.handle_http_error(environ, response.start_response, e)
-        yield response.write(data)
+        if environ:
+            # Get the data from the WSGI handler
+            try:
+                data = worker.app_handler(environ, response.start_response)
+            except Exception as e:
+                # An exception in the handler
+                data = WsgiResponse(environ = environ)
+                cfg.handle_http_error(data, e)
+                data(environ, response.start_response)
+            yield response.write(data)
         yield response
     
     def monitor_init(self, monitor):
