@@ -46,7 +46,7 @@ class WsgiResponse(object):
 Instances are callable using the standard WSGI call::
 
     response = WsgiResponse(200)
-    response(environ,start_response)
+    response(environ, start_response)
     
 
 .. attribute:: status_code
@@ -88,7 +88,7 @@ Instances are callable using the standard WSGI call::
         self.when_ready = Deferred()
         
         if content is None:
-            content = self.get_content()
+            content = self.default_content()
         elif isinstance(content, bytes):
             content = (content,)
         
@@ -97,13 +97,18 @@ Instances are callable using the standard WSGI call::
         if is_streamed(content):
             self._content_generator = content
         else:
-            self.on_content(content)
+            self.content = content
         
     @property
     def logger(self):
         return self.environ['pulsar.actor'].log if self.environ\
                          else default_logger
-                         
+    
+    def default_content(self):
+        '''Called during initialization when the content given is ``None``.
+By default it returns an empty tuple. Overrides if you need to.'''
+        return ()
+    
     def __call__(self, environ, start_response):
         if self.content is None:
             raise ValueError('No content available')
@@ -114,9 +119,6 @@ Instances are callable using the standard WSGI call::
         start_response(self.status, headers)
         return self
         
-    def get_content(self):
-        return ()
-    
     @property
     def response(self):
         return responses.get(self.status_code)
@@ -128,16 +130,7 @@ Instances are callable using the standard WSGI call::
     def __get_content(self):
         return self._content
     def __set_content(self, content):
-        cl = 0
-        if content is None:
-            content = ()
-        else:
-            if isinstance(content,bytes):
-                content = (content,)
-            for c in content:
-                cl += len(c)
-        self.headers['Content-Length'] = str(cl)
-        self._content = content
+        self.on_content(content)
     content =  property(__get_content, __set_content)
     
     def __str__(self):
@@ -155,6 +148,7 @@ This is usually `True` if a generator is passed to the response object."""
         return is_streamed(self.content)
         
     def _generator(self):
+        #Called by the __iter__ method when the response is streamed.
         content = []
         try:
             for b in generate_content(self._content_generator):
@@ -162,14 +156,10 @@ This is usually `True` if a generator is passed to the response object."""
                     content.append(b)
                 else:
                     yield b'' # release the eventloop
-        except:
-            # This is an unhandled exception, It is 500 Server Error
-            self.logger.critical('Unhandled exception during WSGI response',
-                                 exc_info = True)
-            self.status_code = 500
-            content = []
-        self.on_content(content)
-        self._content_generator = None
+        except Exception as e:
+            pulsar.get_actor().cfg.handle_http_error(self, e)
+        else:
+            self.content = content
         for c in self.content:
             yield c
                 
@@ -184,10 +174,21 @@ This is usually `True` if a generator is passed to the response object."""
         return len(self.content)
         
     def on_content(self, content):
+        cl = 0
+        if content is None:
+            content = ()
+        else:
+            if isinstance(content,bytes):
+                content = (content,)
+            for c in content:
+                cl += len(c)
+        self.headers['Content-Length'] = str(cl)
         if self.content_type:
             self.headers['Content-type'] = self.content_type
-        self.content = content
-        self.when_ready.callback(self)
+        self._content_generator = None
+        self._content = content
+        if not self.when_ready.called:
+            self.when_ready.callback(self)
     
     def set_cookie(self, key, **kwargs):
         """
