@@ -1,8 +1,10 @@
 import sys
 import os
 import signal
+import atexit
 from time import time
 import random
+import threading
 from multiprocessing import current_process
 from threading import current_thread
 
@@ -14,15 +16,17 @@ from pulsar.utils.py2py3 import iteritems, itervalues, pickle
 
 from .eventloop import IOLoop
 from .proxy import ActorProxy, ActorMessage, DEFAULT_MESSAGE_CHANNEL
-from .defer import make_async, raise_failure, async_pair
+from .defer import make_async, raise_failure, async_pair, Failure
 from .mailbox import IOQueue, mailbox
 
 
 __all__ = ['is_actor',
+           'get_actor',
+           'send',
            'Actor',
            'ActorMetaClass',
            'ActorBase',
-           'MAIN_THREAD']
+           'is_mainthread']
 
 
 EMPTY_TUPLE = ()
@@ -32,8 +36,45 @@ EMPTY_DICT = {}
 def is_actor(obj):
     return isinstance(obj,Actor)
 
+
+def is_mainthread(thread = None):
+    '''Check if thread is the main thread. If *thread* is not supplied check
+the current thread'''
+    thread = thread if thread is not None else current_thread() 
+    return isinstance(thread, threading._MainThread)
+
+
+def get_actor(thread = None):
+    '''This function will return the actor running the current thread.'''
+    thread = thread if thread is not None else current_thread()
+    return thread.actor
+
+
+def send(target, action, **params):
+    '''Send a message *action* to *target*.
     
-MAIN_THREAD = current_thread()
+:parameter target: the :class:`Actor` id or an :class:`ActorProxy` or name of
+    the target actor receiving the message.
+:parameter action: the action to perform on the remote :class:`Actor`.
+:parameter params: dictionary of parameters to pass to the remote *action*.
+:rtype: an :class:`ActorMessage` which is a :class:`Deferred` and therefore
+    can be used to attach callbacks.
+
+Typical example::
+
+    >>> a = spawn()
+    >>> r = a.add_callback(lambda p: send(p,'ping'))
+    >>> r.result
+    'pong'
+'''
+    actor = get_actor()
+    if isinstance(target, str):
+        tg = actor.get_actor(target)
+    else:
+        tg = target
+    if not tg:
+        raise ValueError('Cannot send message to {0}'.format(target))
+    return tg.send(actor, action, **params)
 
 
 class ActorMetaClass(type):
@@ -91,7 +132,7 @@ by the remote actor calling the function. For example::
 ActorBase = ActorMetaClass('BaseActor',(object,),{})
 
 
-class Actor(ActorBase,LogginMixin):
+class Actor(ActorBase, LogginMixin):
     '''The base class for concurrent programming in pulsar. In computer science,
 the **Actor model** is a mathematical model of concurrent computation that
 treats *actors* as the universal primitives of computation.
@@ -169,12 +210,12 @@ Here ``a`` is actually a reference to the remote actor.
                  on_task = None, ioqueue = None,
                  monitors = None, name = None, params = None,
                  age = 0, pool_timeout = None, ppid = None,
-                 **kwargs):
+                 linked_actors = None, **kwargs):
         # Call on_init
         self.__ppid = ppid
         self._impl = impl
         self.__mailbox = None
-        self._linked_actors = {}
+        self._linked_actors = linked_actors or {}
         self.age = age
         self.nr = 0
         self._pool_timeout = pool_timeout
@@ -309,7 +350,9 @@ their ids.'''
         raise pickle.PicklingError('{0} - Cannot pickle Actor instances'\
                                    .format(self))
     
-    # HOOKS
+    ############################################################################
+    ##    HOOKS
+    ############################################################################
     
     def on_init(self, **kwargs):
         '''The :ref:`actor callback <actor-callbacks>` run once at the
@@ -349,6 +392,16 @@ iteration of the :attr:`Actor.ioloop`.'''
  :rtype: a dictionary of pickable data.'''
         return data
     
+    def on_message(self, message):
+        '''The :ref:`actor callback <actor-callbacks>` run when a new
+:class:`ActorMessage` *message* has been received by the :attr:`inbox`.'''
+        pass
+    
+    def on_message_processed(self, message, result):
+        '''The :ref:`actor callback <actor-callbacks>` run when an
+:class:`ActorMessage` *message* has been processed.'''
+        pass
+    
     @property
     def state(self):
         '''Current state description. One of ``initial``, ``running``,
@@ -382,8 +435,8 @@ mean it is running.'''
             
     def start(self):
         '''Called after forking to start the life of the actor. This is where
-logging is configured, the :attr:`Actor.inbox` and :attr:`Actor.outbox`
-are registered and the :attr:`Actor.ioloop` is initialised and started.'''
+logging is configured, the :attr:`Actor.mailbox` is registered and the
+:attr:`Actor.ioloop` is initialised and started.'''
         if self._state == self.INITIAL:
             ct = current_thread()
             self._state = self.RUN
@@ -394,15 +447,22 @@ are registered and the :attr:`Actor.ioloop` is initialised and started.'''
             self.log.info('Starting')
             # GET REQUESTS EVENT LOOP
             self.__requestloop = self.get_requestloop()
+            # Initialize mailbox. It will also initialize the ioloop
             self.__mailbox = mailbox(self)
             self.__tid = ct.ident
             self.__pid = os.getpid()
-            # Set the ioloop for the thread
             ct.ioloop = self.ioloop
             self.on_start()
             self._run()
     
     def get_requestloop(self):
+        '''Internal function called at the start of the actor. It builds the
+event loop which will consume requests.
+
+If IO loop is based on a IO queue (CPU bounded workers)
+we bind the :meth:`handle_request` to the loop.
+
+This function is overridden by :class:`Monitor` to perform nothing.'''
         ioq = self.ioqueue
         ready = not ioq
         ioimpl = IOQueue(ioq) if ioq else None
@@ -411,6 +471,8 @@ are registered and the :attr:`Actor.ioloop` is initialised and started.'''
                         logger = self.log,
                         name = self.name,
                         ready = ready)
+        # If IO loop is based on a IO queue (CPU bounded workers)
+        # we bind the :meth:`handle_request` to the loop.
         if ioq:
             ioloop.add_handler('request',
                         lambda fd, request : self.handle_request(request),
@@ -419,10 +481,16 @@ are registered and the :attr:`Actor.ioloop` is initialised and started.'''
         return ioloop
     
     def link_actor(self, proxy):
-        self._linked_actors[proxy.aid] = proxy
-        #add the ioqueue handler if needed
-        if proxy == self.arbiter:
-            self.requestloop.ready = True
+        '''Add the *proxy* to the :attr:`linked_actors` dictionary.
+if *proxy* is not a class:`ActorProxy` instance raise an exception.'''
+        if isinstance(proxy, ActorProxy):
+            self._linked_actors[proxy.aid] = proxy
+            #add the ioqueue handler if needed
+            if proxy == self.arbiter:
+                self.requestloop.ready = True
+        elif not isinstance(proxy,Failure):
+            raise ValueError('Received a bad actor proxy. Cannot link.')
+        return proxy
         
     def handle_message(self, sender, message):
         '''Handle a *message* from a *sender*.'''
@@ -431,7 +499,7 @@ are registered and the :attr:`Actor.ioloop` is initialised and started.'''
     def put(self, request):
         '''Put a request into the actor :attr:`ioqueue` if available.'''
         if self.ioqueue:
-            self.log.debug('Put {0} on IO queue'.format(request))
+            self.log.debug('Put {0} into IO queue'.format(request))
             self.ioqueue.put(('request',request))
         else:
             self.log.error("Trying to put a request on task queue,\
@@ -452,7 +520,7 @@ are registered and the :attr:`Actor.ioloop` is initialised and started.'''
 :parameter callable: a pickable, therefore it must be a pickable callable object
     or a function.
 :rtype: a tuple of :class:`Deferred` instances'''
-        res = self.arbiter.send(self,'run',callable)
+        res = self.arbiter.send(self, 'run', callable)
         return async_pair(res)
         
     ############################################################################
@@ -460,23 +528,26 @@ are registered and the :attr:`Actor.ioloop` is initialised and started.'''
     ############################################################################
     
     def stop(self):
-        '''Stop the actor by stopping its :attr:`Actor.ioloop`
+        '''Stop the actor by stopping its :attr:`Actor.requestloop`
 and closing its :attr:`Actor.mailbox`. Once everything is closed
 properly this actor will go out of scope.'''
         if self._state == self.RUN:
             self._state = self.STOPPING
-            self.log.info('stopping')
+            self.log.debug('stopping')
             stp = self.on_stop()
+            #if the on stop method return something we don't close the event
+            #loops. It means they are owned by another actor.
             if not stp:
-                stp = self.requestloop.stop()
-            make_async(stp).add_callback(lambda r : self.close())\
-                           .add_callback(raise_failure)
+                self.log.debug("Stopping request event loop")
+                stp = make_async(self.requestloop.stop()).add_callback(
+                                lambda r : self.mailbox.close())
+            return make_async(stp).add_callback(lambda r : self.close())\
+                                  .add_callback(raise_failure)
         
     def close(self):
         if self.stopping():
             self._state = self.CLOSE
             self.on_exit()
-            self.mailbox.close()
             self.log.info('exited')
             
     # LOW LEVEL API
@@ -528,12 +599,12 @@ actions:
 
         self.on_task()
     
-    def current_thread(self):
-        '''Return the current thread'''
-        return current_thread()
-    
-    def current_process(self):
-        return current_process()
+    #def current_thread(self):
+    #    '''Return the current thread'''
+    #    return current_thread()
+    #
+    #def current_process(self):
+    #    return current_process()
     
     def isprocess(self):
         '''boolean indicating if this is an actor on a child process.'''
@@ -542,14 +613,16 @@ actions:
     def info(self, full = False):
         '''return A dictionary of information related to the actor
 status and performance.'''
-        data = {'name':self.name,
-                'aid':self.aid[:8],
-                'pid':self.pid,
-                'ppid':self.ppid,
-                'thread':self.tid,
-                'process':self.pid,
-                'isprocess':self.isprocess(),
-                'age':self.age}
+        isp = self.isprocess()
+        data = {'name': self.name,
+                'actor_id': self.aid[:8],
+                'ppid': self.ppid,
+                'thread_id': self.tid,
+                'process_id': self.pid,
+                'isprocess': isp,
+                'age': self.age}
+        if isp:
+            data.update(system.system_info(self.pid))
         requestloop = self.requestloop
         data.update({'uptime': time() - requestloop._started,
                      'event_loops': requestloop.num_loops,
@@ -571,12 +644,14 @@ status and performance.'''
         ch.append(request)
         
     def actor_mailbox_address(self, actor, address):
-        '''The ``actor`` register its inbox ``address``.'''
+        '''The ``actor`` register its mailbox ``address``.'''
         if address:
             self.log.debug('Registering actor {0} inbox address {1}'
-                           .format(actor,address))
+                           .format(actor, address))
             actor.address = address
-    actor_mailbox_address.ack = False
+            return True
+        return False
+    #actor_mailbox_address.ack = False
     
     def actor_callback(self, caller, rid, result):
         '''Actor :ref:`remote function <remote-functions>` which sends
@@ -633,8 +708,7 @@ function.'''
         if system.set_proctitle(proc_name):
             self.log.debug('Set process title to {0}'.format(proc_name))
         #system.set_owner_process(cfg.uid, cfg.gid)
-        current = self.current_thread()
-        if current == MAIN_THREAD:
+        if is_mainthread():
             self.log.debug('Installing signals')
             # The default signal handling function in signal
             sfun = getattr(self,'signal',None)
@@ -648,6 +722,7 @@ function.'''
                         signal.signal(sig, func)
                     except ValueError:
                         break
+            #atexit.register(self.stop)
     
     def _run(self):
         '''The run implementation which must be done by a derived class.'''
@@ -666,4 +741,5 @@ function.'''
             
     handle_int  = _signal_stop
     handle_quit = _signal_stop
-    handle_term = _signal_stop
\ No newline at end of file
+    handle_term = _signal_stop
+    handle_break = _signal_stop
\ No newline at end of file
