@@ -7,14 +7,13 @@ from multiprocessing.queues import Empty
 
 import pulsar
 from pulsar.utils import system
-from pulsar.utils.system import IObase
-from pulsar.utils.tools import Pidfile
+from pulsar.utils.tools import Pidfile, gen_unique_id
 from pulsar.utils.py2py3 import itervalues, iteritems
 
 from .impl import actor_impl
-from .actor import Actor
+from .actor import Actor, get_actor, send
 from .monitor import PoolMixin
-from .proxy import ActorCallBacks
+from .proxy import ActorCallBacks, ActorProxyDeferred
 from .defer import ThreadQueue
 
 
@@ -24,23 +23,49 @@ __all__ = ['arbiter','spawn','Arbiter']
 
 
 def arbiter(daemonize = False):
+    '''Obtain the arbiter instance.'''
     arbiter = process_global('_arbiter')
     if not arbiter:
-        arbiter = Arbiter.spawn(Arbiter,impl='monitor',daemonize=daemonize)
+        arbiter = Arbiter.spawn(Arbiter,
+                                impl='monitor',
+                                daemonize=daemonize)
         process_global('_arbiter',arbiter,True)
     return arbiter
     
     
-def spawn(actorcls = None, **kwargs):
-    '''Create a new :class:`Actor` instance and return its proxy in the arbiter
-process domain.
+def spawn(**kwargs):
+    '''Spawn a new :class:`Actor` instance and return an
+:class:`ActorProxyDeferred`. This method
+can be used from any :class:`Actor`. If not in the :class:`Arbiter` domain,
+the method send a request to the :class:`Arbiter` to spawn a new actor, once
+the arbiter creates the actor it returns the proxy to the original caller.
 
-:param actorcls: :class:`Actor` or one of its subclasses.
-:rtype: instance of :class:`ActorProxyMonitor`'''
-    return arbiter().spawn(actorcls or Actor, **kwargs)
+:rtype: an :class:`ActorProxyDeferred`.
+
+A typical usage::
+
+    >>> a = spawn()
+    >>> a.aid
+    'ba42b02b'
+    >>> a.called
+    True
+    >>> p = a.result
+    >>> p.address
+    ('127.0.0.1', 46691)
+    '''
+    aid = gen_unique_id()[:8]
+    kwargs['aid'] = aid
+    actor = get_actor()
+    # The actor is not the Arbiter. We send a message to the Arbiter to spawn
+    # a new Actor
+    if not isinstance(actor, Arbiter):
+        msg = send('arbiter', 'spawn', **kwargs).add_callback(actor.link_actor)
+    else:
+        msg = actor.spawn(**kwargs)
+    return ActorProxyDeferred(aid, msg)
 
 
-class Arbiter(PoolMixin,Actor):
+class Arbiter(PoolMixin, Actor):
     '''The Arbiter is a very special :class:`Actor`. It is used as
 singletone in the main process and it manages one or more
 :class:`Monitor`.  
@@ -94,9 +119,10 @@ Users access the arbiter by the high level api::
         return m
     
     @classmethod
-    def spawn(cls, actorcls, **kwargs):
+    def spawn(cls, actorcls = None, aid = None, **kwargs):
         '''Create a new :class:`Actor` and return its
 :class:`ActorProxyMonitor`.'''
+        actorcls = actorcls or Actor
         arbiter = process_global('_arbiter')
         cls.lock.acquire()
         try:
@@ -106,13 +132,16 @@ Users access the arbiter by the high level api::
                 kwargs['ppid'] = arbiter.ppid
             impl = kwargs.pop('impl',actorcls.DEFAULT_IMPLEMENTATION)
             timeout = max(kwargs.pop('timeout',cls.DEFAULT_ACTOR_TIMEOUT),
-                            cls.MINIMUM_ACTOR_TIMEOUT)
-            actor_maker = actor_impl(impl,actorcls,timeout,arbiter,kwargs)
+                          cls.MINIMUM_ACTOR_TIMEOUT)
+            actor_maker = actor_impl(impl, actorcls, timeout,
+                                     arbiter, aid, kwargs)
             monitor = actor_maker.proxy_monitor()
             # Add to the list of managed actors if this is a remote actor
             if monitor:
                 arbiter.MANAGED_ACTORS[actor_maker.aid] = monitor
                 actor_maker.start()
+                return monitor.on_address.add_callback(
+                                    lambda address: monitor.proxy)
                 return monitor
             else:
                 return actor_maker.actor
@@ -153,6 +182,10 @@ Users access the arbiter by the high level api::
         else:
             super(Arbiter,self).configure_logging(config = config)
  
+    @property
+    def cfg(self):
+        return self.get('cfg')
+    
     ############################################################################
     # OVERRIDE ACTOR HOOKS
     ############################################################################
@@ -185,6 +218,8 @@ Users access the arbiter by the high level api::
         PoolMixin.on_start(self)
         
     def on_task(self):
+        '''Override the :class:`Actor.on_task` callback to perfrom the
+arbiter tasks at every iteration in the event loop.'''
         sig = self._arbiter_task()
         if sig is None:
             self.manage_actors()
@@ -230,7 +265,13 @@ the timeout. Stop the arbiter.'''
     ############################################################################
     # ADDITIONAL REMOTES
     ############################################################################
-        
+    def actor_spawn(self, caller, linked_actors = None, **kwargs):
+        linked_actors = linked_actors if linked_actors is not None else {}
+        linked_actors[caller.aid] = caller.proxy
+        return self.spawn(linked_actors = linked_actors, **kwargs)
+        #p = self.spawn(linked_actors = linked_actors, **kwargs)
+        #return p.on_address.add_callback(lambda address: p.proxy)
+     
     def actor_kill_actor(self, caller, aid):
         '''Remote function for ``caller`` to kill an actor with id ``aid``'''
         a = self.get_actor(aid)
@@ -252,7 +293,7 @@ the timeout. Stop the arbiter.'''
         Initialize the arbiter. Start listening and set pidfile if needed.
         """
         try:
-            self['cfg'].when_ready(self)
+            self.get('cfg').when_ready(self)
         except:
             pass
         try:
