@@ -7,17 +7,21 @@ import logging
 import traceback
 from inspect import isgenerator, isfunction, ismethod, istraceback
 from time import sleep, time
-try:
+from collections import namedtuple
+
+try:    #pragma nocover
     import queue
-except ImportError:
+except ImportError: #pragma nocover
     import Queue as queue
 ThreadQueue = queue.Queue
 
 from pulsar import AlreadyCalledError, DeferredFailure, Timeout,\
                      NOT_DONE, CLEAR_ERRORS
+from pulsar.utils.py2py3 import raise_error_trace, map
 
 
 __all__ = ['Deferred',
+           'MultiDeferred',
            'Failure',
            'DeferredGenerator',
            'SafeAsync',
@@ -34,10 +38,15 @@ __all__ = ['Deferred',
 
 logger = logging.getLogger('pulsar.async.defer')
 
+remote_stacktrace = namedtuple('remote_stacktrace', 'error_class error trace')
+
 
 def is_stack_trace(trace):
-    if isinstance(trace,tuple) and len(trace) == 3:
+    if isinstance(trace,remote_stacktrace):
         return True
+    elif isinstance(trace,tuple) and len(trace) == 3:
+        return istraceback(trace[2]) or\
+                 (trace[2] is None and isinstance(trace[1],trace[0]))
     return False
 
 
@@ -57,6 +66,10 @@ class Failure(object):
         else:
             self.traces = []
             self.append(err)
+            
+    def __repr__(self):
+        return '\n\n'.join(self.format_all())
+    __str__ = __repr__
     
     def append(self, trace):
         '''Add new failure to self.'''
@@ -67,16 +80,28 @@ class Failure(object):
                 self.traces.append(trace)
         return self
     
+    def format_all(self):
+        for exctype, value, tb in self:
+            if istraceback(tb):
+                tb = traceback.format_exception(exctype, value, tb)
+            if tb:
+                yield '\n'.join(tb)
+            else:
+                yield str(value)
+                
     def __getstate__(self):
         traces = []
         for exctype, value, tb in self:
             if istraceback(tb):
                 tb = traceback.format_exception(exctype, value, tb)
-            traces.append((exctype, value, tb))
+            traces.append(remote_stacktrace(exctype, value, tb))
         state = self.__dict__.copy()
         state['traces'] = traces
         return state
-            
+    
+    def __getitem__(self, index):
+        return self.traces[index]
+    
     def __len__(self):
         return len(self.traces)
     
@@ -84,13 +109,19 @@ class Failure(object):
         return iter(self.traces)
     
     def raise_all(self):
-        self.log()
-        N = len(self.traces)
-        if N == 1:
-            raise DeferredFailure('There was one failure during callbacks.')
-        elif N > 1:
-            raise DeferredFailure('There were {0} failures during callbacks.'\
-                                  .format(N))
+        if self.traces and isinstance(self.traces[-1][1],Exception):
+            eclass, error, trace = self.traces.pop()
+            self.log()
+            raise_error_trace(error,trace)
+        else:
+            self.log()
+            N = len(self.traces)
+            if N == 1:
+                raise DeferredFailure(
+                    'There was one failure during callbacks.')
+            elif N > 1:
+                raise DeferredFailure(
+                    'There were {0} failures during callbacks.'.format(N))
     @property
     def trace(self):
         if self.traces:
@@ -126,7 +157,11 @@ def as_failure(data):
     elif is_stack_trace(data):
         return Failure(data)
     elif isinstance(data,Exception):
-        return Failure((data.__class__,data,None))
+        exc_info = sys.exc_info()
+        if data == exc_info[1]:
+            return Failure(exc_info)
+        else:
+            return Failure((data.__class__,data,None))
     
 
 def raise_failure(result):
@@ -261,7 +296,10 @@ program execution. Instead, it should return a Deferred.
         pass
     
     def __repr__(self):
-        return self._description or self.__class__.__name__
+        v = self._description or self.__class__.__name__
+        if self.called:
+            v += ' (called)'
+        return v
     
     def __str__(self):
         return self. __repr__()            
@@ -270,6 +308,10 @@ program execution. Instead, it should return a Deferred.
     def called(self):
         return self._called
     
+    @property
+    def running(self):
+        return self._runningCallbacks
+    
     def pause(self):
         """Stop processing until :meth:`unpause` is called.
         """
@@ -296,28 +338,26 @@ The function takes at most one argument, the result passed to the
         if not self.called or self._runningCallbacks:
             return
         
-        if self.paused:
-            return
-        
-        while self._callbacks:
-            if isinstance(self.result,Failure):
-                if self.result.should_stop:
-                    self.result.raise_all()
-            callback = self._callbacks.pop(0)
-            try:
-                self._runningCallbacks = True
+        if not self.paused:        
+            while self._callbacks:
+                if isinstance(self.result ,Failure):
+                    if self.result.should_stop:
+                        self.result.raise_all()
+                callback = self._callbacks.pop(0)
                 try:
-                    self.result = callback(self.result)
-                finally:
-                    self._runningCallbacks = False
-                if isinstance(self.result, Deferred):
-                    # Add a pause
-                    self.pause()
-                    # Add a callback to the result to resume callbacks
-                    self.result.add_callback(self._continue)
-                    break
-            except:
-                self.result = update_failure(self.result)
+                    self._runningCallbacks = True
+                    try:
+                        self.result = callback(self.result)
+                    finally:
+                        self._runningCallbacks = False
+                    if isinstance(self.result, Deferred):
+                        # Add a pause
+                        self.pause()
+                        # Add a callback to the result to resume callbacks
+                        self.result.add_callback(self._continue)
+                        break
+                except:
+                    self.result = update_failure(self.result)
             
         if isinstance(self.result,Failure):
             if self.result.should_stop:
@@ -340,12 +380,14 @@ this point, :meth:`add_callback` will run the *callbacks* immediately.
 
 :return: the *result* input parameter
 '''
-        if isinstance(result,Deferred):
-            raise ValueError('Received a deferred instance from\
- callback function')
+        #TODO, this is a hack, not working properly yet, but required as
+        #un-handled exception stall the asynchronous engine
+        if isinstance(result, Deferred):
+            raise RuntimeError('Received a deferred instance from '
+                               'callback function')
         if self.called:
-            raise AlreadyCalledError
-        self.result = result
+            raise AlreadyCalledError('already called')
+        self.result = as_failure(result) or result
         self._called = True
         self._run_callbacks()
         return self.result
@@ -411,7 +453,60 @@ in :meth:`start`.'''
             except:
                 self.callback(sys.exc_info())
         
+
+class MultiDeferred(Deferred):
+    '''A :class:`Deferred` which depends on multiple other :class:`Deferred`.'''
+    def __init__(self, **kwargs):
+        self._underlyings = set()
+        self._results = []
+        self._locked = False
+        super(MultiDeferred,self).__init__(**kwargs)
+    
+    def lock(self):
+        if self._locked:
+            raise RuntimeError(
+                        'MultiDeferred cannot be locked twice.')
+        self._locked = True
+        if not self._underlyings:
+            self._finish()
+        return self
             
+    def add(self, d):
+        if not isinstance(d, Deferred):
+            raise ValueError('Expected a Deferred received %s' % d)
+        if self._locked:
+            raise RuntimeError(
+                        'MultiDeferred cannot add a dependent once locked.')
+        self._results.append(d)
+        if d not in self._underlyings:
+            self._underlyings.add(d)
+            d.add_callback_args(self._underlying_done, d)
+        return self
+    
+    def update(self, deferred):
+        '''Update the multideferred with an iterable over :class:`Deferred`'''
+        for d in deferred:
+            self.add(d)
+        return self
+    
+    def _underlying_done(self, result, d):
+        self._underlyings.remove(d)
+        if self._locked and not self._underlyings and not self.called:
+            self._finish()
+        return result
+    
+    def _finish(self):
+        if not self._locked:
+            raise RuntimeError('MultiDeferred cannot finish until completed.')
+        if self._underlyings:
+            raise RuntimeError('MultiDeferred cannot finish whilst waiting for '
+                               'dependents %r' % self._underlyings)
+        if self.called:
+            raise RuntimeError('MultiDeferred done before finishing.')
+        result = [r.result for r in self._results]
+        return self.callback(result)            
+        
+        
 class DeferredGenerator(Deferred):
     '''A :class:`Deferred` for a generator (iterable) over deferred.
 The callback will occur once the generator has stopped
@@ -456,10 +551,14 @@ generator.'''
                 else:
                     d = make_async(result, description = self._description)
                     if d.called:
-                        consume = not self._should_stop(d.result)
-                    else:
-                        # the deferred is not ready.
-                        return d.add_callback(self._resume)\
+                        # the deferred was paused
+                        if d.paused:
+                            d = d.result
+                        else:
+                            consume = not self._should_stop(d.result)
+                            continue
+                    # the deferred is not ready.
+                    return d.add_callback(self._resume)\
                                 .start(self._ioloop, self._timeout)
         
         if consume:
