@@ -2,11 +2,11 @@ import logging
 import traceback
 import time
 import signal
-import threading
 import errno
 import bisect
 import socket
 
+from pulsar import HaltServer
 from pulsar.utils.system import IObase, IOpoll, close_on_exec, platform, Waker
 from pulsar.utils.tools import gen_unique_id
 from pulsar.utils.log import Synchronized
@@ -204,7 +204,13 @@ For example, a :class:`pulsar.Actor` add itself to the event loop tasks
 so that it can perform its tasks at each event loop. Check the
 :meth:`pulsar.Actor` method.'''
         for task in self._loop_tasks:
-            task()
+            try:
+                task()
+            except HaltServer:
+                raise
+            except:
+                self.log.critical('Unhandled exception in loop task',
+                                  exc_info = True)
         
     def start(self):
         """Starts the I/O loop.
@@ -295,7 +301,8 @@ so that it can perform its tasks at each event loop. Check the
                                 # Happens when the client closes the connection
                                 pass
                             else:
-                                self.log.error("Exception in I/O handler for fd %d",
+                                self.log.error(
+                                    "Exception in I/O handler for fd %d",
                                               fd, exc_info=True)
                         except:
                             self.log.error("Exception in I/O handler for fd %d",
@@ -316,7 +323,6 @@ unit tests), you can start and stop the event loop like this::
 :meth:`start` will return after async_method has run its callback,
 whether that callback was invoked before or after ioloop.start.'''
         if self.running():
-            self.log.debug("Stopping event loop")
             self._running = False
             self.wake()
         return self._on_exit
