@@ -17,6 +17,7 @@ from .defer import make_async, raise_failure, Failure
 __all__ = ['mailbox','Mailbox','IOQueue','Empty','Queue']
 
 crlf = b'\r\n'
+msg_separator = 3*crlf
 
 
 def mailbox(actor, address = None):
@@ -35,36 +36,6 @@ otherwise a queue is used.'''
         return mailbox
 
 
-def process_message(receiver, sender, message):
-    '''Process *message* received by *receiver* by performing the message
-action. If an acknowledgment is required, send back the result using
-the receiver eventloop.'''
-    ack = message.ack
-    try:
-        func = receiver.actor_functions.get(message.action,None)
-        if func:
-            ack = getattr(func,'ack',True)
-            result = func(receiver, sender, *message.args, **message.kwargs)
-        else:
-            result = receiver.handle_message(sender, message)
-    except Exception as e:
-        result = Failure(sys.exc_info())
-        if receiver.log:
-            receiver.log.critical('Error while processing message: {0}.'\
-                              .format(e), exc_info=True)
-    finally:
-        if ack:
-            if sender:
-                # Acknowledge the sender with the result.
-                make_async(result).start(receiver.ioloop)\
-                .add_callback(
-                        lambda res : sender.send(receiver, 'callback',\
-                                                 message.rid, res))\
-                .add_callback(raise_failure)
-            else:
-                receiver.log.error('message "{0}" from an unknown actor "{1}"'\
-                                   .format(message,message.sender))
-
 def serverSocket():
     '''Create the inbox, a TCP socket ready for
 accepting messages from other actors.'''
@@ -95,7 +66,7 @@ send messages to a :class:`SocketServerMailbox`.'''
         return self.sock.fileno()
         
     def put(self, request):
-        request = pickle.dumps(request) + crlf
+        request = pickle.dumps(request) + msg_separator
         try:
             return self.sock.send(request)
         except socket.error as e:
@@ -108,6 +79,8 @@ send messages to a :class:`SocketServerMailbox`.'''
 
 
 class SocketServerClient(object):
+    '''A client of :class:`Mailbox`. An instance of this class is created
+when a new connection is mad with :class:`Mailbox`.'''
     __slots__ = ('sock','buffer')
     def __init__(self, sock):
         sock.setblocking(True)
@@ -142,13 +115,13 @@ class SocketServerClient(object):
             toclose = True
             
         while buffer:
-            p = buffer.find(crlf)
+            p = buffer.find(msg_separator)
             if p >= 0:
                 msg = buffer[:p]
-                del buffer[0:p+2]
+                del buffer[0:p+len(msg_separator)]
                 try:
                     data = pickle.loads(bytes(msg))
-                except pickle.UnpicklingError:
+                except (pickle.UnpicklingError, EOFError):
                     actor.log.error('Could not unpickle message',
                                     exc_info = True)
                     continue
@@ -162,9 +135,34 @@ class SocketServerClient(object):
     
     
 class Mailbox(threading.Thread):
-    '''An inbox for :class:`Actor` instances. If an address is provided,
-the communication is implemented using a socket, otherwise a unidirectional
-pipe is created.'''
+    '''A socket mailbox for :class:`Actor` instances. If the *actor* is a
+CPU bound worker (an actor which communicate with its monitor via a message
+queue), the class:`Mailbox` will create its own :class:`IOLoop`.
+
+A :class:`Mailbox` is created during an :class:`Actor` startup, in the
+actor process domain.
+
+.. attribute:: actor
+
+    The :class:`Actor` which uses this :class:`Mailbox` to send and receive
+    :class:`ActorMessage`.
+    
+.. attribute:: sock
+
+    The socket which is initialised during construction with a random local
+    address.
+    
+.. attribute:: ioloop
+
+    The :class:`IOLoop` used by this mailbox for asynchronously sending and
+    receiving :class:`ActorMessage`. There are two possibilities:
+    
+    * The mailbox shares the eventloop with the :attr:`actor`. This is the
+      most common case, when the actor is not a CPU bound worker.
+    * The mailbox has its own eventloop. This is the case when the :attr:`actor`
+      is a CPU-bound worker, and uses its main event loop for communicating
+      with a queue rather than with a socket.
+'''
     def __init__(self, actor):
         self.actor = actor
         self.sock = serverSocket()
@@ -203,10 +201,17 @@ pipe is created.'''
         return self.sock.fileno()
     
     def register(self, actor):
-        '''Register *actor* with this mailbox by sending its socket address.
-        '''
+        '''This class:`Mailbox` register with a remote *actor* by
+        
+* addint *actor* to the mailbox actor linked actors dictionary
+* sending its socket address.
+
+:parameter actor: an :class:`ActorProxy` to register with.
+:rtype: an :class:`ActorMessage`.
+'''
         self.actor.link_actor(actor)
-        return actor.send(self.actor,'mailbox_address',self.address)
+        msg = actor.send(self.actor, 'mailbox_address', self.address)
+        return msg.add_callback(lambda r: self.actor)
         
     def on_message(self, fd, events):
         '''Handle the message by parsing it and invoking
@@ -239,6 +244,7 @@ pipe is created.'''
     def close(self):
         self.unregister(self.actor)
         if self.__hasloop:
+            self.ioloop.log.debug('Stop event loop for {0}'.format(self))
             self.ioloop.stop()
         if self.sock:
             self.ioloop.log.debug('shutting down {0}'.format(self))
@@ -262,16 +268,48 @@ If the message needs acknowledgment, send the result back.'''
         actor = self.actor
         sender = actor.get_actor(message.sender)
         receiver = actor.get_actor(message.receiver)
-        #if not sender or not receiver:
-        #    #if not sender:
-        #    #    actor.log.warn('message "{0}" from an unknown actor "{1}"'\
-        #    #                  .format(message,message.sender))
         if not receiver:
             actor.log.warn('message "{0}" for an unknown actor "{1}"'\
                               .format(message,message.receiver))
-        else:
-            return process_message(receiver,sender,message)
-            
+            return
+        
+        ack = message.ack
+        try:
+            receiver.on_message(message)
+        except:
+            pass
+        try:
+            func = receiver.actor_functions.get(message.action, None)
+            if func:
+                ack = getattr(func, 'ack', True)
+                result = func(receiver, sender, *message.args, **message.kwargs)
+            else:
+                result = receiver.handle_message(sender, message)
+        except Exception as e:
+            result = Failure(sys.exc_info())
+            if receiver.log:
+                receiver.log.critical('Unhandled error while processing\
+ message: {0}.'.format(e), exc_info=True)
+        finally:
+            if ack:
+                if sender:
+                    # Acknowledge the sender with the result.
+                    make_async(result).start(receiver.ioloop)\
+                    .add_callback(lambda res: self._send_callback(
+                            sender, receiver, message, res))\
+                    .add_callback(raise_failure)
+                else:
+                    receiver.log.error('message "{0}" from an unknown actor\
+ "{1}". Cannot acknowledge message.'.format(message,message.sender))
+            try:
+                receiver.on_message_processed(message, result)
+            except:
+                pass
+    
+    def _send_callback(self, sender, receiver, message, result):
+        '''Send the callback to the server. It returns nothing since the
+message does not acknowledge'''
+        sender.send(receiver, 'callback', message.rid, result)
 
 class MailBoxMonitor(object):
     
@@ -320,7 +358,6 @@ class IOQueue(object):
     '''Epoll like class for a IO based on queues.
 The interface is the same as the python epoll_ implementation.
 
-
 .. _epoll: http://docs.python.org/library/select.html#epoll-objects'''
     def __init__(self, queue):
         self._queue = queue
@@ -351,7 +388,6 @@ The interface is the same as the python epoll_ implementation.
             event = self._queue.get(timeout = timeout)
         except (Empty,IOError,TypeError,EOFError):
             return self._empty
-        
         return (event,)
     
     def waker(self):
