@@ -1,7 +1,7 @@
 import sys
 from time import time
 
-from pulsar import create_connection
+from pulsar import create_connection, CannotCallBackError
 from pulsar.utils.py2py3 import iteritems
 from pulsar.utils.log import LocalMixin
 from pulsar.utils.tools import gen_unique_id
@@ -10,6 +10,7 @@ from .defer import Deferred, is_async, make_async, raise_failure, Failure
 from .mailbox import mailbox
 
 __all__ = ['ActorMessage',
+           'ActorProxyDeferred',
            'ActorProxy',
            'ActorProxyMonitor',
            'get_proxy',
@@ -69,7 +70,9 @@ class ActorCallBacks(Deferred):
 
 class ActorMessage(Deferred):
     '''A message class which travels from :class:`Actor` to
-:class:`Actor` to perform a specific action.
+:class:`Actor` to perform a specific *action*. :class:`ActorMessage`
+are not directly initialized using the constructor, instead they are
+created by :meth:`ActorProxy.send` method.
 
 .. attribute:: sender
 
@@ -94,6 +97,11 @@ class ActorMessage(Deferred):
 .. attribute:: ack
 
     ``True`` if the message needs acknowledgment
+    
+.. attribute:: rid
+
+    :class:`ActorMessage` request id. This id used for managing remote
+    callbacks, if :attr:`ack` is ``True``.
 '''
     MESSAGES = {}
     
@@ -112,8 +120,14 @@ class ActorMessage(Deferred):
             self.MESSAGES[self.rid] = self
         
     def __str__(self):
-        return '[{0}] - {1} {2} {3}'.format(self.rid,self.sender,self.action,
-                                            self.receiver)
+        return '[{0} - from {1}] - {3} {2}'.format(self.rid,self.sender,
+                                                   self.action,self.receiver)
+    
+    def add_callback(self, callback, raise_on_error = False):
+        if not self.ack:
+            raise CannotCallBackError('Cannot add callback to "{0}".\
+ It does not acknowledge'.format(self))
+        return super(ActorMessage,self).add_callback(callback, raise_on_error)
     
     def __repr__(self):
         return self.__str__()
@@ -136,7 +150,30 @@ class ActorMessage(Deferred):
             r.callback(result)
             
             
+class ActorProxyDeferred(Deferred):
+    '''A :class:`Deferred` for an :class:`ActorProxy`. This instance will be
+obtain and :class:`ActorProxy` result once the remote :class:`Actor` is fully
+functional.
 
+.. attribute:: aid
+
+    The the remote :attr:`Actor` id
+'''
+    def __init__(self, aid, msg):
+        super(ActorProxyDeferred,self).__init__(rid = aid)
+        self.aid = aid
+        self.proxy = None
+        self._msg = msg.add_callback(self._store_proxy)
+        
+    def _store_proxy(self, proxy):
+        self.proxy = proxy
+        return self.callback(proxy)
+    
+    def __str__(self):
+        return '{0}({1})'.format(self.__class__,self.aid)
+    __repr__ = __str__
+    
+    
 class ActorProxy(LocalMixin):
     '''This is an important component in pulsar concurrent framework. An
 instance of this class behaves as a proxy for a remote `underlying` 
@@ -196,7 +233,7 @@ action ``notify`` with parameter ``"hello there!"``.
         '''Callback from the :class:`Arbiter` when the remote underlying actor
 has registered its inbox address.
 
-:parameter address: the address for the undrlying remote :attr:`Arbiter.inbox`
+:parameter address: the address for the underlying remote :attr:`Arbiter.inbox`
 '''
         self.__address = address
         m = self.local.pop('mailbox',None)
@@ -232,13 +269,13 @@ Send an :class:`ActorMessage` to the underlying actor
 (the receiver). This is the low level function call for
 communicating between actors.
 
-:parameter sender: the actor sending the message.
+:parameter sender: :class:`Actor` sending the message.
 :parameter action: the action of the message. If not provided,
     the message will be broadcasted by the receiving actor,
     otherwise a specific action will be performed.
     Default ``None``.
-:parameter args: non key-valued part of parameters of message body.
-:parameter kwargs: key-valued part of parameters of message body.
+:parameter args: non positional arguments of message body.
+:parameter kwargs: key-valued arguments of message body.
 :parameter ack: If ``True`` the receiving actor will send a callback.
     If the action is provided and available, this parameter will be overritten.
 :rtype: an instance of :class:`ActorMessage`.
@@ -253,7 +290,7 @@ If there is no inbox either, abort the message passing and log a critical error.
  mailbox available.'.format(self))
             return
         
-        msg = self.message(sender,action,*args,**kwargs)
+        msg = self.message(sender, action, *args, **kwargs)
         try:
             mailbox.put(msg)
             return msg
@@ -285,10 +322,11 @@ object including, aid (actor id), timeout and mailbox size.'''
         
 
 class ActorProxyMonitor(ActorProxy):
-    '''A specialised :class:`pulsar.ActorProxy` class which contains additional
+    '''A specialised :class:`ActorProxy` class which contains additional
 information about the remote underlying :class:`pulsar.Actor`. Unlike the
 :class:`pulsar.ActorProxy` class, instances of this class are not pickable and
-therefore remain in the process where they have been created.
+therefore remain in :class:`Arbiter` domain, which is the
+process where they have been created.
 
 .. attribute:: impl
 
@@ -322,6 +360,10 @@ therefore remain in the process where they have been created.
     def pid(self):
         return self.impl.pid
     
+    @property
+    def proxy(self):
+        return ActorProxy(self)
+    
     def is_alive(self):
         '''True if underlying actor is alive'''
         return self.impl.is_alive()
