@@ -137,18 +137,19 @@ attribute by exposing the :attr:`Setting.name` as attribute.
     keys are given by the :attr:`Setting.name` attribute.
 '''
     def __init__(self, description = None, epilog = None,
-                 app = None, include=None, exclude = None):
+                 version = None, app = None, include=None,
+                 exclude = None):
         self.settings = make_settings(app,include,exclude)
         self.description = description or 'Pulsar server'
         self.epilog = epilog or 'Have fun!'
+        self.version = version or __version__
         
     def __getstate__(self):
         return self.__dict__.copy()
     
     def __setstate__(self, state):
-        self.__dict__['settings'] = state['settings']
-        self.__dict__['description'] = state['description']
-        self.__dict__['epilog'] = state['epilog']
+        for k,v in state.items():
+            self.__dict__[k] = v
     
     def __getattr__(self, name):
         if name not in self.settings:
@@ -182,7 +183,7 @@ settings via the :meth:`Setting.add_argument`.
         parser = argparse.ArgumentParser(**kwargs)
         parser.add_argument('--version',
                             action='version',
-                            version = __version__)
+                            version = self.version)
         setts = self.settings
         sorter = lambda x: (setts[x].section, setts[x].order)
         for k in sorted(setts,key=sorter):
@@ -261,10 +262,7 @@ in the global ``KNOWN_SETTINGS`` list.'''
         
         
 # This works for Python 2 and Python 3
-BaseSettings =  SettingMeta('BaseSettings', (object, ), {})
-
-
-class Setting(BaseSettings):
+class Setting(SettingMeta('BaseSettings', (object,), {'virtual': True})):
     '''A configuration parameter for pulsar. Parameters can be specified
 on the command line or on a config file.'''
     virtual = True
@@ -302,7 +300,11 @@ accepting one positional argument, the value to validate.'''
         self.desc = self.desc or self.short
         if self.app and not self.section:
             self.section = self.app
-            
+    
+    def __getstate__(self):
+        d = self.__dict__.copy()
+        return d
+    
     def __str__(self):
         return '{0} ({1})'.format(self.name,self.value)
     __repr__ = __str__
@@ -517,7 +519,7 @@ class HttpParser(Setting):
     action = "store_true"
     default = False
     desc = """\
-        Set the python parser as default HTTP parser.    
+    Set the python parser as default HTTP parser.    
         """
     
     def on_start(self):
@@ -799,3 +801,15 @@ class WorkerExit(Setting):
         The callable needs to accept one variable for the
         the just-exited Worker.
         """
+
+
+class WorkerTask(Setting):
+    name = "worker_task"
+    section = "Server Hooks"
+    validator = validate_callable(1)
+    type = "callable"
+    default = staticmethod(def_worker_exit)
+    desc = """Called at every event loop by the worker.
+
+        The callable needs to accept one variable for the Worker.
+        """
\ No newline at end of file
