@@ -39,10 +39,16 @@ class ConnectionError(PulsarInternetException):
 class DeferredFailure(PulsarException):
     pass
 
+
 class AlreadyCalledError(PulsarException):
     '''Raised when a :class:`Deferred` instance receives more than
 one :meth:`Deferred.callback`.'''
 
+
+class CannotCallBackError(PulsarException):
+    pass
+
+
 class AlreadyRegistered(PulsarException):
     pass
 
@@ -51,9 +57,9 @@ class NotRegisteredWithServer(PulsarException):
 
 
 class BadHttpRequest(PulsarInternetException):
-    status = 500
-    def __init__(self, status = None, reason = ''):
-        self.status = status or self.status
+    status_code = 500
+    def __init__(self, status_code = None, reason = ''):
+        self.status_code = status_code or self.status_code
         super(BadHttpRequest,self).__init__(reason)
         
         
